#include "config.h"

#include "xmms/i18n.h"
#include <glib.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <signal.h>
#include <unistd.h>
#include <stdio.h>

#include <string.h>

#include "xmms/plugin.h"
#include "libxmms/configfile.h"
#include "libxmms/xmmsctrl.h"
#include "libxmms/formatter.h"

static void init(void);
static void cleanup(void);
static void configure(void);
static gint timeout_func(gpointer);
static gint timeout_tag = 0;
static gint previous_song = -1, previous_length = -2;
static gchar *cmd_line = NULL;
static gboolean possible_pl_end;
static gchar *cmd_line_end;

static void *configure_win = NULL, *configure_vbox;
static void *cmd_entry, *cmd_end_entry;

GeneralPlugin sc_gp =
{
	NULL,			/* handle */
	NULL,			/* filename */
	-1,			/* xmms_session */
	NULL,			/* Description */
	init,
	NULL,
	configure,
	cleanup,
};

GeneralPlugin *get_gplugin_info(void)
{
	sc_gp.description = g_strdup_printf(_("Song Change %s"), VERSION);
	return &sc_gp;
}

static void read_config(void)
{
	ConfigFile *cfgfile;

	g_free(cmd_line);
	g_free(cmd_line_end);
	cmd_line = NULL;
	cmd_line_end = NULL;

	if ((cfgfile = xmms_cfg_open_default_file()) != NULL)
	{
		xmms_cfg_read_string(cfgfile, "song_change", "cmd_line", &cmd_line);
		xmms_cfg_read_string(cfgfile, "song_change", "cmd_line_end", &cmd_line_end);
		xmms_cfg_free(cfgfile);
	}
}

static void init(void)
{
	read_config();

	previous_song = -1;
	previous_length = -2;
}

static void cleanup(void)
{
	timeout_tag = 0;
	if (cmd_line)
		g_free(cmd_line);
	if (cmd_line_end)
		g_free(cmd_line_end);
	cmd_line = NULL;
	cmd_line_end = NULL;
	signal(SIGCHLD, SIG_DFL);
}

static void configure_ok_cb(gpointer data)
{}

static void configure(void)
{
	read_config();
}

static void bury_child(int signal)
{
	waitpid(-1, NULL, WNOHANG);
}

static void execute_command(gchar *cmd)
{
	gchar *argv[4] = {"/bin/sh", "-c", NULL, NULL};
	gint i;
	argv[2] = cmd;
	signal(SIGCHLD, bury_child);
	if (fork() == 0)
	{
		/* We don't want this process to hog the audio device etc */
		for (i = 3; i < 255; i++)
			close(i);
		execv("/bin/sh", argv);
	}
}

/*
 * escape_shell_chars()
 *
 * Escapes characters that are special to the shell inside double quotes.
 */

static gchar * escape_shell_chars(gchar * string)
{
	const gchar *special = "$`\"\\"; /* Characters to escape */
	gchar *in = string, *out;
	gchar *escaped;
	gint num = 0;

	while (*in != '\0')
		if (strchr(special, *in++))
			num++;

	escaped = g_malloc(strlen(string) + num + 1);

	in = string;
	out = escaped;

	while (*in != '\0')
	{
		if (strchr(special, *in))
			*out++ = '\\';
		*out++ = *in++;
	}
	*out = '\0';

	return escaped;
}

static gint timeout_func(gpointer data)
{
	gint pos, length, rate, freq, nch;
	gchar *str, *shstring = NULL, *temp, numbuf[16];
	gboolean playing, run_end_cmd = FALSE;
	Formatter *formatter;

	playing = xmms_remote_is_playing(sc_gp.xmms_session);
	pos = xmms_remote_get_playlist_pos(sc_gp.xmms_session);
	length = xmms_remote_get_playlist_time(sc_gp.xmms_session, pos);
	
	/* Format codes:
	 *
	 *   F - frequency (in hertz)
	 *   c - number of channels
	 *   f - filename (full path)
	 *   l - length (in milliseconds)
	 *   n - name
	 *   r - rate (in bits per second)
	 *   s - name (since everyone's used to it)
	 *   t - playlist position (%02d)
	 */
	if (pos != previous_song || length != previous_length)
	{
		previous_song = pos;
		previous_length = length;
		if (cmd_line && strlen(cmd_line) > 0)
		{
			formatter = xmms_formatter_new();
			str = xmms_remote_get_playlist_title(sc_gp.xmms_session, pos);
			if (str)
			{
				temp = escape_shell_chars(str);
				xmms_formatter_associate(formatter, 's', temp);
				xmms_formatter_associate(formatter, 'n', temp);
				g_free(str);
				g_free(temp);
			}
			str = xmms_remote_get_playlist_file(sc_gp.xmms_session, pos);
			if (str)
			{
				temp = escape_shell_chars(str);
				xmms_formatter_associate(formatter, 'f', temp);
				g_free(str);
				g_free(temp);
			}
			sprintf(numbuf, "%02d", pos + 1);
			xmms_formatter_associate(formatter, 't', numbuf);
			sprintf(numbuf, "%d", length);
			xmms_formatter_associate(formatter, 'l', numbuf);
			xmms_remote_get_info(sc_gp.xmms_session, &rate, &freq, &nch);
			sprintf(numbuf, "%d", rate);
			xmms_formatter_associate(formatter, 'r', numbuf);
			sprintf(numbuf, "%d", freq);
			xmms_formatter_associate(formatter, 'F', numbuf);
			sprintf(numbuf, "%d", nch);
			xmms_formatter_associate(formatter, 'c', numbuf);
			shstring = xmms_formatter_format(formatter, cmd_line);
			xmms_formatter_destroy(formatter);
		}
	}

	if (playing)
	{
		gint playlist_length = xmms_remote_get_playlist_length(sc_gp.xmms_session);
		if (pos + 1 == playlist_length)
			possible_pl_end = TRUE;
		else
			possible_pl_end = FALSE;
	}
	else if (possible_pl_end)
	{
		if (pos == 0 && cmd_line_end && strlen(cmd_line_end) > 0)
			run_end_cmd = TRUE;
		possible_pl_end = FALSE;
	}

	if (shstring)
	{
		execute_command(shstring);
		g_free(shstring); /* FIXME: This can possibly be freed too early */
	}

	if (run_end_cmd)
		execute_command(cmd_line_end);

	return TRUE;
}
