/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *  Copyright (C) 1999,2000  Hvard Kvlen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdarg.h>
#include "mpg123.h"

static void *window = NULL;
static void *filename_entry, *id3_frame;
static void *title_entry, *artist_entry, *album_entry, *year_entry, *comment_entry;
static void *genre_combo;
static void *mpeg_level, *mpeg_bitrate, *mpeg_samplerate, *mpeg_flags;
static void *mpeg_fileinfo;

static GList *genre_list = NULL;
static gchar *current_filename = NULL;

extern gchar *mpg123_filename;
extern gint mpg123_bitrate, mpg123_frequency, mpg123_layer, mpg123_lsf, mpg123_mode;
extern gboolean mpg123_stereo, mpg123_mpeg25;

#define MAX_STR_LEN 100

static void set_entry_tag(void * entry, gchar * tag, gint length)
{}

static void get_entry_tag(void * entry, gchar * tag, gint length)
{}

static gint find_genre_id(gchar * text)
{
	gint i;

	for (i = 0; i < GENRE_MAX; i++)
	{
		if (!strcmp(mpg123_id3_genres[i], text))
			return i;
	}
	if (text[0] == '\0')
		return 0xff;
	return 0;
}

static gint genre_comp_func(gconstpointer a, gconstpointer b)
{
	return strcasecmp(a, b);
}

static void save_cb(void * w, gpointer data)
{}

static void label_set_text(void * label, char *str, ...)
{}

static void remove_id3_cb(void * w, gpointer data)
{}


static void set_mpeg_level_label(gboolean mpeg25, gint lsf, gint layer)
{
	if (mpeg25)
		label_set_text(mpeg_level, "MPEG 2.5, layer %d", layer);
	else
		label_set_text(mpeg_level, "MPEG %d, layer %d", lsf + 1, layer);
}

static gchar* channel_mode_name(int mode)
{
	static const gchar *channel_mode[] =
		{N_("Stereo"), N_("Joint stereo"),
		 N_("Dual channel"), N_("Single channel")};
	if (mode < 0 || mode > 3)
		return "";
	return gettext(channel_mode[mode]);
}

void trim(char *trimme, int len)
{
   int i;
   
   /* Place the null terminator correctly. */
   for(len;len>=0;len--)
   {
      if(trimme[len] == '\0')
         break;
      
      if(trimme[len] == ' ')
      {
         trimme[len]='\0';
         if(trimme[len-1] != ' ')
            break;
      }
   }
}

static void file_info_http(char *filename)
{}

static void file_info_http_vanilla(char *filename, FileInfo *info)
{
   char *title;
   char *station;
   int i;
   
   if (mpg123_filename && !strcmp(filename, mpg123_filename) &&
       mpg123_bitrate != 0)
	{  
      info->bitrate = mpg123_bitrate;
      info->sampling_frequency = mpg123_frequency;
      strcpy(info->channels,channel_mode_name(mpg123_mode));

      title = get_current_title();
      station = get_current_station();

      if(title != NULL)
      {
         strncpy(info->title,title,256);
         trim(info->title,256);
      }
      
      if(station != NULL)
      {
         strncpy(info->station,station,256);
         trim(info->station,256);
      }
   }
}

void mpg123_file_info_box(char *filename)
{}

void mpg123_file_info(char *filename, FileInfo *info)
{
   gint i;
	struct id3v1tag_t tag;
	FILE *fh;
	gchar *tmp, *title;
	const gchar *emphasis[] = {N_("None"), N_("50/15 ms"),
				   "", N_("CCIT J.17")};
	const gchar *bool_label[] = {N_("No"), N_("Yes")};

#ifdef ENABLE_NLS
	{
		int i;

		for (i=0; i < 4; i++)
			if (*emphasis[i])
				emphasis[i] = gettext(emphasis[i]);
		for (i=0; i < 2; i++)
			bool_label[i] = gettext(bool_label[i]);
	}
#endif

   if (current_filename)
		g_free(current_filename);
	current_filename = g_strdup(filename);
   
   /* If this is a stream, handle it differently. */
   if (!strncasecmp(filename, "http://", 7))
	{
		file_info_http_vanilla(filename, info);
		return;
	}

   if ((fh = fopen(current_filename, "rb")) != NULL)
	{
		guint32 head;
		unsigned char tmp[4];
		struct frame frm;
		gboolean id3_found = FALSE;

		fseek(fh, -sizeof (tag), SEEK_END);
      
		if (fread(&tag, 1, sizeof (tag), fh) == sizeof (tag))
		{
			if (!strncmp(tag.tag, "TAG", 3))
			{
				id3_found = TRUE;
            
            strncpy(info->title,tag.title,30);
            trim(info->title,30);
            strncpy(info->artist,tag.artist,30);
            trim(info->artist,30);
            strncpy(info->album,tag.album,30);
            trim(info->album,30);
            strncpy(info->year,tag.year,4);
            trim(info->year,4);
            strncpy(info->comment,tag.comment,30);
            trim(info->comment,30);
            if(tag.genre <= GENRE_MAX)
               strcpy(info->genre,mpg123_id3_genres[tag.genre]);
         }
		}
		rewind(fh);
		if (fread(tmp, 1, 4, fh) != 4)
		{
			fclose(fh);
			return;
		}
		head = ((guint32) tmp[0] << 24) | ((guint32) tmp[1] << 16) |
         ((guint32) tmp[2] << 8) | (guint32) tmp[3];
		while (!mpg123_head_check(head))
		{
			head <<= 8;
			if (fread(tmp, 1, 1, fh) != 1)
			{
				fclose(fh);
				return;
			}
			head |= tmp[0];
		}
		if (mpg123_decode_header(&frm, head))
		{
			guchar *buf;
			gdouble tpf;
			gint pos;
			XHEADDATA xing_header;
			guint32 num_frames;

			buf = g_malloc(frm.framesize + 4);
			fseek(fh, -4, SEEK_CUR);
			fread(buf, 1, frm.framesize + 4, fh);
			xing_header.toc = NULL;
			tpf = mpg123_compute_tpf(&frm);
			pos = ftell(fh);
			fseek(fh, 0, SEEK_END);
			if(mpg123_get_xing_header(&xing_header, buf))
			{
            num_frames = xing_header.frames;
            info->bitrate = 0;
			}
         else
         {
            num_frames = ((ftell(fh) - pos - (id3_found ? 128 : 0)) / mpg123_compute_bpf(&frm)) + 1;
            info->bitrate = tabsel_123[frm.lsf][frm.lay - 1][frm.bitrate_index];
         }

         strcpy(info->channels,channel_mode_name(frm.mode));
         info->sampling_frequency = mpg123_freqs[frm.sampling_frequency];
         strcpy(info->error_protection,bool_label[frm.error_protection]);
         strcpy(info->copyright,bool_label[frm.copyright]);
         strcpy(info->original,bool_label[frm.original]);
         strcpy(info->emphasis,emphasis[frm.emphasis]);
         info->frames = num_frames;
         info->filesize = ftell(fh);
         
			g_free(buf);
		}
		fclose(fh);
	}
}
