/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "xmms.h"
#include <sys/ipc.h>
#include <ctype.h>
#include <sys/mount.h>

/*
 * find_file_recursively() by Jrg Schuler Wed, 17 Feb 1999 23:50:52
 * +0100 Placed under GPL version 2 or (at your option) any later
 * version
 */
/*
 * find <file> in directory <dirname> or subdirectories.  return
 * pointer to complete filename which has to be freed by calling
 * "g_free()" after use. Returns NULL if file could not be found.
 */
gchar *find_file_recursively(const char *dirname, const char *file)
{
	DIR *dir;
	struct dirent *dirent;
	gchar *result;
	gchar *found;
	struct stat buf;

	if ((dir = opendir(dirname)) != NULL)
	{
		while ((dirent = readdir(dir)) != NULL)
		{
			/* Don't scan "." and ".." */
			if (strcmp(dirent->d_name, ".") == 0 || strcmp(dirent->d_name, "..") == 0)
				continue;
			/* We need this in order to find out if file is directory */
			found = g_strdup_printf("%s/%s", dirname, dirent->d_name);
			if (stat(found, &buf) != 0)
			{	/* Error occured */
				g_free(found);
				closedir(dir);
				return NULL;
			}
			if (S_ISDIR(buf.st_mode))
			{	/* Found directory -- descend into it */
				result = find_file_recursively(found, file);
				if (result != NULL)
				{	/* Found file there -- exit */
					g_free(found);
					closedir(dir);
					return result;
				}
			}
			else
			{	/* Normal file -- maybe just what we are looking for? */
				if (!strcasecmp(dirent->d_name, file))
				{	/* Yes! */
					if (strlen(found) > FILENAME_MAX)
					{	/* No good! File + path too long */
						g_free(found);
						closedir(dir);
						return NULL;
					}
					else
					{
						closedir(dir);
						return found;
					}
				}
			}
			g_free(found);
		}
		closedir(dir);
	}
	return NULL;
}

void del_directory(const char *dirname)
{
	DIR *dir;
	struct dirent *dirent;
	char *file;

	if ((dir = opendir(dirname)) != NULL)
	{
		while ((dirent = readdir(dir)) != NULL)
		{
			if (strcmp(dirent->d_name, ".") && strcmp(dirent->d_name, ".."))
			{
				file = g_strdup_printf("%s/%s", dirname, dirent->d_name);
				if (unlink(file) == -1)
					if (errno == EISDIR)
						del_directory(file);
				g_free(file);
			}
		}
		closedir(dir);
	}
	rmdir(dirname);
}

char *read_ini_string(const char *filename, const char *section, const char *key)
{
	FILE *file;
	char *buffer, *ret_buffer = NULL;
	int found_section = 0, found_key = 0, off = 0, len = 0;
	struct stat statbuf;

	if (!filename)
		return NULL;

	if ((file = fopen(filename, "r")) != NULL)
	{
		stat(filename, &statbuf);
		buffer = (char *) g_malloc(statbuf.st_size);
		fread(buffer, 1, statbuf.st_size, file);
		while (!found_key && off < statbuf.st_size)
		{
			while ((buffer[off] == '\r' || buffer[off] == '\n' ||
				buffer[off] == ' ' || buffer[off] == '\t') &&
			       off < statbuf.st_size)
				off++;
			if (off >= statbuf.st_size)
				break;
			if (buffer[off] == '[')
			{
				off++;
				if (off >= statbuf.st_size)
					break;
				if (off < statbuf.st_size - strlen(section))
				{
					if (!strncasecmp(section, &buffer[off], strlen(section)))
					{
						off += strlen(section);
						if (off >= statbuf.st_size)
							break;
						if (buffer[off] == ']')
							found_section = 1;
						else
							found_section = 0;
						off++;
						if (off >= statbuf.st_size)
							break;
					}
					else
						found_section = 0;
				}
				else
					found_section = 0;

			}
			else if (found_section)
			{
				if (off < statbuf.st_size - strlen(key))
				{
					if (!strncasecmp(key, &buffer[off], strlen(key)))
					{
						off += strlen(key);
						while ((buffer[off] == ' ' || buffer[off] == '\t') && off < statbuf.st_size)
							off++;
						if (off >= statbuf.st_size)
							break;
						if (buffer[off] == '=')
						{
							off++;
							while ((buffer[off] == ' ' || buffer[off] == '\t') && off < statbuf.st_size)
								off++;
							if (off >= statbuf.st_size)
								break;
							len = 0;
							while (buffer[off + len] != '\r' && buffer[off + len] != '\n' && buffer[off + len] != ';' && off + len < statbuf.st_size)
								len++;
							ret_buffer = (char *) g_malloc(len + 1);
							strncpy(ret_buffer, &buffer[off], len);
							ret_buffer[len] = '\0';
							off += len;
							found_key = 1;
						}
					}
				}
			}
			while (buffer[off] != '\r' && buffer[off] != '\n' && off < statbuf.st_size)
				off++;
		}
		g_free(buffer);
		fclose(file);
	}
	return ret_buffer;
}

GArray *string_to_garray(const gchar * str)
{
	GArray *array;
	gint temp;
	const gchar *ptr = str;
	gchar *endptr;

	array = g_array_new(FALSE, TRUE, sizeof (gint));
	for (;;)
	{
		temp = strtol(ptr, &endptr, 10);
		if (ptr == endptr)
			break;
		g_array_append_val(array, temp);
		ptr = endptr;
		while (!isdigit(*ptr) && (*ptr) != '\0')
			ptr++;
		if (*ptr == '\0')
			break;
	}
	return (array);
}

GArray *read_ini_array(const gchar * filename, const gchar * section, const gchar * key)
{
	gchar *temp;
	GArray *a;

	if ((temp = read_ini_string(filename, section, key)) == NULL)
		return NULL;
	a = string_to_garray(temp);
	g_free(temp);
	return a;
}

void glist_movedown(GList * list)
{
	gpointer temp;

	if (g_list_next(list))
	{
		temp = list->data;
		list->data = g_list_next(list)->data;
		g_list_next(list)->data = temp;
	}
}

void glist_moveup(GList * list)
{
	gpointer temp;

	if (g_list_previous(list))
	{
		temp = list->data;
		list->data = g_list_previous(list)->data;
		g_list_previous(list)->data = temp;

	}
}

static gint util_find_compare_func(gconstpointer a, gconstpointer b)
{
	return strcasecmp(a, b);
}

static int int_compare_func(gconstpointer a, gconstpointer b)
{
	if (GPOINTER_TO_INT(a) < GPOINTER_TO_INT(b))
		return -1;
	if (GPOINTER_TO_INT(a) > GPOINTER_TO_INT(b))
		return 1;
	else
		return 0;
}

char *get_ps2_mc()
{
   /* Unmount both directories just incase. */
   umount_ps2_mc();

   /* Mount the first available device. */
   if(mount("/dev/ps2mc00", "/mnt/mc00", "ps2mcfs", MS_SYNCHRONOUS, NULL) == 0)
   {
      return "/mnt/mc00";
   }
   else if(mount("/dev/ps2mc10","/mnt/mc10","ps2mcfs",MS_SYNCHRONOUS,NULL) == 0)
   {
      return "/mnt/mc10";
   }
   else //No memory card, so return home directory.
      return g_get_home_dir();
}

void umount_ps2_mc()
{
   umount("/mnt/mc00");
   umount("/mnt/mc10");
}


